/***********************************************************************
 * Macro qui assigne les deux dimensions de l'indice de          *******
 * dfavorisation qubcois et des indices locaux (quartiles     *******
 * pour les RLS et les rgions administratives), la zone         *******
 *  partir du code postal et du code municipal  tout fichier   *******
 * administratif comprenant au moins le code postal.             *******
 * On utilise une version simplifie du FCCP+ produit par        *******
 * Statistique Canada. En tout, le programme fait appel          *******
 *  3 fichiers de conversion.                                   *******
 *  - WCF du FCCP+                                               *******
 *  - PCCFDUPS du FCCP+                                          *******
 *  - PCCFUNIQ du FCCP+                                          *******
 * Paramtres de la macro                                        *******
 *  - IN: fichier SAS d'entre auquel on dsire ajouter          *******
 *        l'indice de dfavorisation (materiel et social)        *******
 *  - PCODE: nom de la variable du code postal  6 positions     *******
 *  - MUNIC: nom de la variable du code municipal. Par dfaut,   *******
 *           0 = l'attribution ne se fait qu' partir du code    *******
 *           postal seulement                                    *******
 *  - OUT: nom du fichier SAS de sortie identique  IN mais avec *******
 *         les nouvelles variables AD et indices de              *******
 *         dfavorisation                                        *******
 * Auteur: Denis Hamel, INSPQ                                    *******
 *         Philippe Gamache, INSPQ                               *******
 * Dernire modification: 19 novembre 2015                       *******
 * Modifications: Ajout des quintiles rgionaux			         *******
 ***********************************************************************/

filename fcp "E:\Projets\Indice dfavorisation\Indice 2006\Assignation\Qubec\Novembre2015\fcp.txt";
filename pccfuniq "E:\Projets\Indice dfavorisation\Indice 2006\Assignation\Qubec\Novembre2015\fccpindiceuniq.txt";
filename pccfdups "E:\Projets\Indice dfavorisation\Indice 2006\Assignation\Qubec\Novembre2015\fccpindicedouble.txt";
filename munic "E:\Projets\Indice dfavorisation\Indice 2006\Assignation\Qubec\Novembre2015\munic2011.txt";
libname i2006 "E:\Projets\Indice dfavorisation\Indice 2006\Assignation\Qubec\Novembre2015";

data i2006.wcf;
infile fcp;
input  munic 1-12 pcode $ 13-18 propmunic 19-23 propcp 24-28 quintmat 29 quintsoc 30 centmat 31-33 centsoc 34-36 diffmatcp 37 diffsoccp 38
       diffmatmunic 39 diffsocmunic 40 fortcp 41 fortmunic 42 zone 44 rss 46-47 QuintMatRSS 48 QuintSocRSS 49 QuartMatRSS 50 quartsocRSS 51 
       quartmatRLS 52 quartsocRLS 53 rls $ 54-57 frac 58-60 propmunic_RLS 61-65 propcp_RLS 66-70 quartmatclsc 71 quartsocclsc 72 clsc $ 73-77 frac_clsc 78-80 
       propmunic_clsc 81-85 propcp_clsc 86-90 first 91 css 92;
run;

data i2006.pccfuniq;
infile pccfuniq;
input  munic 1-12 pcode $ 13-18 propmunic 19-23 propcp 24-28 quintmat 29 quintsoc 30 centmat 31-33 centsoc 34-36 diffmatcp 37 diffsoccp 38
       diffmatmunic 39 diffsocmunic 40 fortcp 41 fortmunic 42 zone 44 rss 46-47 QuintMatRSS 48 QuintSocRSS 49 QuartMatRSS 50 quartsocRSS 51 
       quartmatRLS 52 quartsocRLS 53 rls $ 54-57 frac 58-60 propmunic_RLS 61-65 propcp_RLS 66-70 quartmatclsc 71 quartsocclsc 72 clsc $ 73-77 frac_clsc 78-80 
       propmunic_clsc 81-85 propcp_clsc 86-90 first 91 css 92;
run;

data i2006.pccfdups;
infile pccfdups;
input  munic 1-12 pcode $ 13-18 propmunic 19-23 propcp 24-28 quintmat 29 quintsoc 30 centmat 31-33 centsoc 34-36 diffmatcp 37 diffsoccp 38
       diffmatmunic 39 diffsocmunic 40 fortcp 41 fortmunic 42 zone 44 rss 46-47 QuintMatRSS 48 QuintSocRSS 49 QuartMatRSS 50 quartsocRSS 51 
       quartmatRLS 52 quartsocRLS 53 rls $ 54-57 frac 58-60 propmunic_RLS 61-65 propcp_RLS 66-70 quartmatclsc 71 quartsocclsc 72 clsc $ 73-77 frac_clsc 78-80 
       propmunic_clsc 81-85 propcp_clsc 86-90 first 91 css 92;
run;

data i2006.munic;
  infile munic;
  input munic 1-7 mun2011 8-14;
run;

%macro presence(var=);
%global presence;
%let dsid=%sysfunc(open(fichin,i));
    %if &dsid>0 %then
       %do;
	     %let presence=%sysfunc(varnum(&dsid,&var));
		 /*** la macrovariable presence = 0 si la variable est absente du fichier ****/
	   %end;
%put &presence;
%let rc = %sysfunc(close(&dsid));
%mend presence;

/**** Ne pas changer la macro qui suit *****/
%macro assignation2006(in,pcode,munic,out);

data wcf;
  set i2006.wcf;
  nuniq = _n_;
run;

data pccfuniq;
  set i2006.pccfuniq;
  nuniq = _n_;
run;

data pccfdups;
  set i2006.pccfdups;
  nuniq = _n_;
run;

proc sort data=i2006.munic;by munic;run;
proc sort data=wcf;by munic;run;
proc sort data=pccfuniq;by munic;run;
proc sort data=pccfdups;by munic;run;

data wcf;
  merge i2006.munic(in=in2) wcf(in=in1);
  by munic;
  if in1*in2 then munic=mun2011;
  else if in2 then delete;
run;

data pccfuniq;
  merge i2006.munic(in=in2) pccfuniq(in=in1);
  by munic;
  if in1*in2 then munic=mun2011;
  else if in2 then delete;
run;

data pccfdups;
  merge i2006.munic(in=in2) pccfdups(in=in1);
  by munic;
  if in1*in2 then munic=mun2011;
  else if in2 then delete;
run;

data fichin;
  %if &munic ne 0 %then %do; length munic 8.; %end;
  set &in;
  numero=_n_;
  nba=ranuni(766);
  pcode=compress(&pcode);
  %if &munic ne 0 %then %do; munic=2400000+mod(&munic,2400000); %end;
run;

%presence(var=quintmat);

%if &presence ne 0 %then %do;
   data fichin;
     set fichin;
     where zone in (.,0);
   run;
%end;

data fichin;
  set fichin;
  keep numero nba pcode %if &munic ne 0 %then %do; munic %end;;
run;

%if &munic ne 0 %then %do;

proc sort data=i2006.munic;by munic;run;
proc sort data=fichin;by munic;run;

data fichin;
  merge i2006.munic(in=in2) fichin(in=in1);
  by munic;
  if in1*in2 then munic=mun2011;
  else if in2 then delete;
run;


/*********************************************************
  * Assignation des indices de dfavorisation  partir ***
  * du code postal et du code de la municipalit       ***
  * Fichier FCCP+                                      ***
  *******************************************************/

/*** Avec WCF *****/
/*** RLS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, wcf.propmunic_RLS, wcf.rls, wcf.frac, wcf.quartmatRLS, wcf.quartsocRLS, 
         wcf.nuniq, wcf.first
  from fichin, wcf
  where fichin.pcode=wcf.pcode and fichin.munic=wcf.munic
  order by numero,nuniq; 
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
    end;
  if find = 0 then do;
    cumul=cumul+(propmunic_RLS/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 


/*** CLSC ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, fichin.numero, fichin.nba, wcf.propmunic_CLSC, wcf.clsc, wcf.frac_clsc, wcf.quartmatclsc, wcf.quartsocclsc, 
         wcf.nuniq, wcf.first
  from fichin, wcf
  where fichin.pcode=wcf.pcode and fichin.munic=wcf.munic
  order by numero,nuniq; 
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
    end;
  if find = 0 then do;
    cumul=cumul+(propmunic_CLSC/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 


/*** RSS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.munic, numero, nba, wcf.first, wcf.propmunic, wcf.diffmatmunic, wcf.diffsocmunic, wcf.fortcp, wcf.fortmunic, 
         wcf.quintmat, wcf.quintsoc, wcf.centmat, wcf.centsoc, wcf.zone, wcf.css, wcf.rss, wcf.quartmatRSS, wcf.quartsocRSS, 
         wcf.quintmatRSS, wcf.quintsocRSS, wcf.nuniq
  from fichin, wcf
  where fichin.pcode=wcf.pcode and fichin.munic=wcf.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  where first = 1;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=test2;by numero;run;
proc sort data=test3;by numero;run;
proc sort data=fichin;by numero;run; 

data appmunic1 nonapp1;
  merge fichin(in=in1) test1(in=in2a) test2(in=in2b) test3(in=in2c);
  by numero;
  if in1*in2a*in2b*in2c then output appmunic1;
  else if in1 then output nonapp1;
run;

/*** Avec PCCFUNIQ *****/
/*** RLS ***/
proc sort data=nonapp1;by pcode munic;run;
proc sort data=pccfuniq; by pcode munic;run;

proc sql;
  create table test as
  select nonapp1.pcode, nonapp1.munic, nonapp1.numero, nonapp1.nba, pccfuniq.propmunic_RLS, pccfuniq.nuniq, pccfuniq.rls, pccfuniq.frac, pccfuniq.css, 
         pccfuniq.quartmatRLS, pccfuniq.quartsocRLS
  from nonapp1, pccfuniq
  where nonapp1.pcode=pccfuniq.pcode and nonapp1.munic=pccfuniq.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par RLS ***/
data test1;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_RLS/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 


/*** CLSC ***/
proc sort data=nonapp1;by pcode munic;run;
proc sort data=pccfuniq; by pcode munic;run;

proc sql;
  create table test as
  select nonapp1.pcode, nonapp1.munic, nonapp1.numero, nonapp1.nba, pccfuniq.propmunic_CLSC, pccfuniq.nuniq, pccfuniq.clsc, pccfuniq.frac_clsc, 
         pccfuniq.css, pccfuniq.quartmatclsc, pccfuniq.quartsocclsc
  from nonapp1, pccfuniq
  where nonapp1.pcode=pccfuniq.pcode and nonapp1.munic=pccfuniq.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par RLS ***/
data test2;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_CLSC/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 


  /*** RSS ***/
proc sql;
  create table test as
  select nonapp1.pcode, nonapp1.munic, numero, nba, pccfuniq.propmunic, pccfuniq.diffmatmunic, pccfuniq.diffsocmunic, pccfuniq.fortcp, pccfuniq.fortmunic, 
         pccfuniq.css, pccfuniq.quintmat, pccfuniq.quintsoc, pccfuniq.centmat, pccfuniq.centsoc, pccfuniq.zone,pccfuniq.nuniq, pccfuniq.rss, 
         pccfuniq.quartmatRSS, pccfuniq.quartsocRSS, pccfuniq.quintmatRSS, pccfuniq.quintsocRSS
  from nonapp1, pccfuniq
  where nonapp1.pcode=pccfuniq.pcode and nonapp1.munic=pccfuniq.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=test2;by numero;run;
proc sort data=test3;by numero;run;
proc sort data=nonapp1;by numero;run; 

data appmunic2 nonapp2;
  merge nonapp1(in=in1) test1(in=in2a) test2(in=in2b) test3(in=in2c);
  by numero;
  if in1*in2a*in2b*in2c then output appmunic2;
  else if in1 then output nonapp2;
run;

/*** Avec PCCFDUPS *****/
/*** RLS ***/
proc sql;
  create table test as
  select nonapp2.pcode, nonapp2.munic, nonapp2.numero, nonapp2.nba, pccfdups.propmunic_RLS, pccfdups.nuniq, pccfdups.rls, pccfdups.frac,
         pccfdups.quartmatRLS, pccfdups.quartsocRLS
  from nonapp2, pccfdups
  where nonapp2.pcode=pccfdups.pcode and nonapp2.munic=pccfdups.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1;
				   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_RLS/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

/*** CLSC ***/
proc sql;
  create table test as
  select nonapp2.pcode, nonapp2.munic, nonapp2.numero, nonapp2.nba, pccfdups.propmunic_clsc, pccfdups.nuniq, pccfdups.clsc, pccfdups.frac_clsc,
         pccfdups.quartmatclsc, pccfdups.quartsocclsc
  from nonapp2, pccfdups
  where nonapp2.pcode=pccfdups.pcode and nonapp2.munic=pccfdups.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1;
				   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic_CLSC/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

/*** RSS ***/
proc sql;
  create table test as
  select nonapp2.pcode, nonapp2.munic, nonapp2.numero, nonapp2.nba, pccfdups.first, pccfdups.propmunic, pccfdups.diffmatmunic, pccfdups.diffsocmunic, 
         pccfdups.fortcp, pccfdups.nuniq, pccfdups.css, pccfdups.fortmunic, pccfdups.quintmat, pccfdups.quintsoc, pccfdups.centmat, pccfdups.centsoc,
         pccfdups.zone, pccfdups.rss, pccfdups.quartmatRSS, pccfdups.quartsocRSS, pccfdups.quintmatRSS, pccfdups.quintsocRSS
  from nonapp2, pccfdups
  where nonapp2.pcode=pccfdups.pcode and nonapp2.munic=pccfdups.munic
  order by numero,nuniq;
quit;

proc sort data=test;by pcode munic numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  where first = 1;
  by pcode munic numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propmunic/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=test2;by numero;run;
proc sort data=test3;by numero;run;
proc sort data=nonapp2;by numero;run; 

data appmunic3 fichin;
  merge nonapp2(in=in1) test1(in=in2a) test2(in=in2b) test3(in=in2c);
  by numero;
  if in1*in2a*in2b*in2c then output appmunic3;
  else if in1 then output fichin;
run;

%end;

/*********************************************************
  * Assignation des indices de dfavorisation  partir ***
  * du code postal seulement                           ***
  * Fichier FCCP+                                      ***
  *******************************************************/

/*** Avec WCF *****/
data wcf2;
  set wcf;
run;

/*** RLS ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, wcf2.propcp_RLS, wcf2.nuniq, wcf2.rls, wcf2.frac, wcf2.quartmatRLS, wcf2.quartsocRLS
  from fichin, wcf2
  where fichin.pcode=wcf2.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1;
				   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_RLS/100);
	if nba < cumul then do; 
      find=1;
      output; 
    end;
  end;
run; 


/*** CLSC ***/
proc sql;
  create table test as
  select fichin.pcode, fichin.numero, fichin.nba, wcf2.propcp_clsc, wcf2.nuniq, wcf2.clsc, wcf2.frac_clsc, wcf2.quartmatclsc, wcf2.quartsocclsc
  from fichin, wcf2
  where fichin.pcode=wcf2.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1;
				   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_clsc/100);
	if nba < cumul then do; 
      find=1;
      output; 
    end;
  end;
run; 


/*** RSS ***/

proc sql;
  create table test as
  select fichin.pcode, fichin.numero, nba, wcf2.first, wcf2.propcp, wcf2.diffmatcp, wcf2.diffsoccp, wcf2.fortmunic, wcf2.fortcp, 
         wcf2.nuniq, wcf2.quintmat, wcf2.quintsoc, wcf2.centmat, wcf2.centsoc, wcf2.css, wcf2.zone, wcf2.rss, 
         wcf2.quartmatRSS, wcf2.quartsocRSS, wcf2.quintmatRSS, wcf2.quintsocRSS
  from fichin, wcf2
  where fichin.pcode=wcf2.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  where first = 1;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=test2;by numero;run;
proc sort data=test3;by numero;run;
proc sort data=fichin;by numero;run; 

data appcp1 nonapp3 ;
  merge fichin(in=in1) test1(in=in2A) test2(in=in2B) test3(in=in2c);
  by numero;
  if in1*in2a*in2b*in2c then output appcp1;
  else if in1 then output nonapp3;
run;

 /*** Avec PCCFUINIQ *****/
proc sort data=nonapp3;by pcode;run;
proc sort data=pccfuniq;by pcode;run;


/*** RLS ***/
proc sql;
  create table test as
  select nonapp3.pcode, nonapp3.numero, nonapp3.nba, pccfuniq.propcp_rls, pccfuniq.rls, pccfuniq.frac, pccfuniq.nuniq, 
         pccfuniq.quartmatRLS, pccfuniq.quartsocRLS
  from nonapp3, pccfuniq
  where nonapp3.pcode=pccfuniq.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par RLS ***/
data test1;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_RLS/100);
    if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 


/*** CLSC ***/
proc sql;
  create table test as
  select nonapp3.pcode, nonapp3.numero, nonapp3.nba, pccfuniq.propcp_CLSC, pccfuniq.CLSC, pccfuniq.frac_CLSC, pccfuniq.nuniq, 
         pccfuniq.quartmatCLSC, pccfuniq.quartsocCLSC
  from nonapp3, pccfuniq
  where nonapp3.pcode=pccfuniq.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par RLS ***/
data test2;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_CLSC/100);
    if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 


/*** RSS ***/
proc sql;
  create table test as
  select nonapp3.pcode, nonapp3.numero, nba, pccfuniq.propcp, pccfuniq.diffmatcp, pccfuniq.diffsoccp, pccfuniq.fortmunic, pccfuniq.fortcp, 
         pccfuniq.nuniq, pccfuniq.quintmat, pccfuniq.quintsoc, pccfuniq.centmat, pccfuniq.centsoc, pccfuniq.css, pccfuniq.zone, pccfuniq.rss, 
         pccfuniq.quartmatRSS, pccfuniq.quartsocRSS, pccfuniq.quintmatRSS, pccfuniq.quintsocRSS
  from nonapp3, pccfuniq
  where nonapp3.pcode=pccfuniq.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=test2;by numero;run;
proc sort data=test3;by numero;run;
proc sort data=nonapp3;by numero;run; 

data appcp2 nonapp4 ;
  merge nonapp3(in=in1) test1(in=in2A) test2(in=in2B) test3(in=in2c);
  by numero;
  if in1*in2a*in2b*in2c then output appcp2;
  else if in1 then output nonapp4;
run;


/*** Avec PCCFDUPS *****/
data pccfdup2;
  set pccfdups;
run;

/*** RLS ***/
proc sql;
  create table test as
  select nonapp4.pcode, nonapp4.numero, nonapp4.nba, pccfdup2.propcp_RLS, pccfdup2.nuniq, pccfdup2.rls, pccfdup2.frac, 
         pccfdup2.quartmatRLS, pccfdup2.quartsocRLS
  from nonapp4, pccfdup2
  where nonapp4.pcode=pccfdup2.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test1;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1;
				   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_RLS/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 


/*** CLSC ***/
proc sql;
  create table test as
  select nonapp4.pcode, nonapp4.numero, nonapp4.nba, pccfdup2.propcp_clsc, pccfdup2.nuniq, pccfdup2.clsc, pccfdup2.frac_clsc, 
         pccfdup2.quartmatclsc, pccfdup2.quartsocclsc
  from nonapp4, pccfdup2
  where nonapp4.pcode=pccfdup2.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test2;
  set test;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1;
				   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp_clsc/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

/*** RSS ***/
proc sql;
  create table test as
  select nonapp4.pcode, nonapp4.numero, nba, pccfdup2.first, pccfdup2.propcp, pccfdup2.diffmatcp, pccfdup2.diffsoccp, pccfdup2.fortmunic, pccfdup2.fortcp, 
         pccfdup2.nuniq, pccfdup2.quintmat, pccfdup2.quintsoc, pccfdup2.centmat, pccfdup2.centsoc, pccfdup2.css, pccfdup2.zone, pccfdup2.rss, 
         pccfdup2.quartmatRSS, pccfdup2.quartsocRSS, pccfdup2.quintmatRSS, pccfdup2.quintsocRSS
  from nonapp4, pccfdup2
  where nonapp4.pcode=pccfdup2.pcode
  order by numero,nuniq;
quit;

proc sort data=test;by pcode numero;run;

/*** Choix de l'indice de dfavorisation si plus d'un indice par code postal ***/
data test3;
  set test;
  where first = 1;
  by pcode numero;
  retain cumul find direct;
  if first.numero then do; 
    find=0; 
    cumul=0; 
    if last.numero then direct=1; 
                   else direct=0;
  end;
  if find = 0 then do;
    cumul=cumul+(propcp/100);
	if nba < cumul then do; 
      find=1; 
      output; 
    end;
  end;
run; 

proc sort data=test1;by numero;run;
proc sort data=test2;by numero;run;
proc sort data=test3;by numero;run;
proc sort data=nonapp4;by numero;run; 

data appcp3 nonapp5 ;
  merge nonapp4(in=in1) test1(in=in2A) test2(in=in2B) test3(in=in2c);
  by numero;
  if in1*in2a*in2b*in2c then output appcp3;
  else if in1 then output nonapp5;
run;

data tout;
  set %if &munic ne 0 %then %do; appmunic1(in=in1) appmunic2(in=in2) appmunic3(in=in3) %end;
	  appcp1(in=in4) appcp2(in=in5) appcp3(in=in6) nonapp5(in=in7);
  if in7 then direct=0;
  if in5 then do; 
    direct=1;
    diffmatcp=0; 
    diffsoccp=0;
  end;
  %if &munic ne 0 %then %do; 
    if in1 then base=1;
    else if in2 then do; 
      direct=1; 
      diffmatmunic=0; 
      diffsocmunic=0; 
      base=2;
	end;
	else if in3 then do;
      base=3;
	  diffmat=diffmatmunic;
	  diffsoc=diffsocmunic;
	end;
  %end;
  if in4 then base=1;
  else if in5 then base=2;
  else if in6 then base=3;
  if diffmatcp ne . then diffmat=diffmatcp;
  if diffsoccp ne . then diffsoc=diffsoccp;

  if in7=1 then do; 
    quintmat = 0; quintsoc = 0;
    quartmatRLS=0; quartsocRLS=0;
	quartmatclsc=0; quartsocclsc=0;
    quartmatRSS=0; quartsocRSS=0;
	quintmatRSS=0; quintsocRSS=0;
	zone = 0;
  end;
  if quintmat=0 then groupe=6;
  else if quintmat=. then groupe=5;
  else if direct=1 then groupe=1;
  else if diffmat=0 and diffsoc=0 then groupe=2;
  else if fortcp=1 or fortmunic=1 then groupe=3;
  else groupe=4;

  if propmunic_RLS ne . then propRLS=propmunic_RLS;
  else if propcp_RLS ne . then propRLS=propcp_RLS;
  else propRLS=100;

  if propmunic_clsc ne . then propclsc=propmunic_clsc;
  else if propcp_clsc ne . then propclsc=propcp_clsc;
  else propclsc=100;

  if propmunic ne . then prop=propmunic;
  else if propcp ne . then prop=propcp;
  else prop=100;

  keep numero base groupe quintmat quintsoc quartmatRSS quartsocRSS quintmatRSS quintsocRSS quartmatRLS quartsocRLS quartmatclsc quartsocclsc zone rss css;
run;

proc sort; by numero; run; 

data fichin;
  set &in;
  numero=_n_;
run;

data tout;
  retain QuintMat QuintSoc RSS QuintMatRSS QuintSocRSS QuartMatRSS QuartSocRSS QuartMatRLS QuartSocRLS QuartMatCLSC QuartSocCLSC zone base groupe css numero;
  set tout;
run;

data &out;
  merge fichin tout;
  by numero;
  DATE = '20151119';
  format groupe fgroupe. base fbase.;
  drop numero;
run;

proc datasets library=work;
  delete corr2 wcf2 pccfuniq2 pccfdups2 test test1 test2 test3 fichin tout appmunic1 appmunic2 appmunic3 appmunic4 appcp1 
         appcp2 appcp3 appcp4 nonapp1 nonapp2 nonapp3 nonapp4 nonapp5 nonapp6 nonapp7 pccfdup2; 
quit;
%mend assignation2006;

proc format;
  value fgroupe 1="Assignation CP - Indice unique"
  		        2="Assignation non unique mais tous les AD mnent au mme indice"
				3="Assignation non unique mais avec prdominance d'un AD  plus de 90%"
				4="Autre assignation non unique"
				5="Appartient  un AD sans indice"
				6="Aucun appariement CP - AD - Indice";
  value fbase 1="WCF"
			  2="PCFF unique"
			  3="PCCF dups";
 run;

/****************************************************************
* Exemple: %assignation(naiss96, cp, cod_mun, naiss96v5); 		*
* - Le fichier en entre est WORK.NAISS96              			*
* - La variable pour le code postal est CP             			*
* - La variable pour le code municipal est COD_MUN     			*
*   N.B.: on doit mettre 0 (zro) si on ne veut pas faire   	*
*   l'assignation  l'aide du code municipal     				*
* - Le fichier rsultant est WORK.NAISS96V2. Il est identique 	*
*   au fichier d'entre, mais avec l'indice de dfavorisation  	*
*   en quintiles, en quartiles, la rgion et la variable zone  	*
* - NOTE: Les rsultats peuvent varier si l'ordre du fichier  	*
*   d'entre n'est pas la mme.                					*
****************************************************************/ 

%assignation2006(in=dec04,pcode=CPRESDCD,munic=CMUNRESD,out=dec04v2);
